@@ -1,83 +1,123 @@
 #!/usr/bin/env python
 """Support classes for automated testing.
 
-This module contains three parts:
+* `AsyncTestCase` and `AsyncHTTPTestCase`:  Subclasses of unittest.TestCase
+  with additional support for testing asynchronous (`.IOLoop` based) code.
 
-* `AsyncTestCase`/`AsyncHTTPTestCase`:  Subclasses of unittest.TestCase
-  with additional support for testing asynchronous (IOLoop-based) code.
-
-* `LogTrapTestCase`:  Subclass of unittest.TestCase that discards log output
-  from tests that pass and only produces output for failing tests.
+* `ExpectLog` and `LogTrapTestCase`: Make test logs less spammy.
 
 * `main()`: A simple test runner (wrapper around unittest.main()) with support
   for the tornado.autoreload module to rerun the tests when code changes.
-
-These components may be used together or independently.  In particular,
-it is safe to combine AsyncTestCase and LogTrapTestCase via multiple
-inheritance.  See the docstrings for each class/function below for more
-information.
 """
 
-from __future__ import absolute_import, division, with_statement
+from __future__ import absolute_import, division, print_function, with_statement
 
-from cStringIO import StringIO
 try:
+    from tornado import gen
     from tornado.httpclient import AsyncHTTPClient
     from tornado.httpserver import HTTPServer
     from tornado.simple_httpclient import SimpleAsyncHTTPClient
     from tornado.ioloop import IOLoop
+    from tornado import netutil
 except ImportError:
     # These modules are not importable on app engine.  Parts of this module
     # won't work, but e.g. LogTrapTestCase and main() will.
     AsyncHTTPClient = None
+    gen = None
     HTTPServer = None
     IOLoop = None
+    netutil = None
     SimpleAsyncHTTPClient = None
-from tornado.stack_context import StackContext, NullContext
-from tornado.util import raise_exc_info
-import contextlib
+from tornado.log import gen_log
+from tornado.stack_context import ExceptionStackContext
+from tornado.util import raise_exc_info, basestring_type
+import functools
 import logging
 import os
+import re
 import signal
+import socket
 import sys
-import time
-import unittest
+
+try:
+    from cStringIO import StringIO  # py2
+except ImportError:
+    from io import StringIO  # py3
+
+# Tornado's own test suite requires the updated unittest module
+# (either py27+ or unittest2) so tornado.test.util enforces
+# this requirement, but for other users of tornado.testing we want
+# to allow the older version if unitest2 is not available.
+try:
+    import unittest2 as unittest
+except ImportError:
+    import unittest
 
 _next_port = 10000
 
 
 def get_unused_port():
-    """Returns a (hopefully) unused port number."""
+    """Returns a (hopefully) unused port number.
+
+    This function does not guarantee that the port it returns is available,
+    only that a series of get_unused_port calls in a single process return
+    distinct ports.
+
+    **Deprecated**.  Use bind_unused_port instead, which is guaranteed
+    to find an unused port.
+    """
     global _next_port
     port = _next_port
     _next_port = _next_port + 1
     return port
 
 
+def bind_unused_port():
+    """Binds a server socket to an available port on localhost.
+
+    Returns a tuple (socket, port).
+    """
+    [sock] = netutil.bind_sockets(0, 'localhost', family=socket.AF_INET)
+    port = sock.getsockname()[1]
+    return sock, port
+
+
 class AsyncTestCase(unittest.TestCase):
-    """TestCase subclass for testing IOLoop-based asynchronous code.
+    """`~unittest.TestCase` subclass for testing `.IOLoop`-based
+    asynchronous code.
 
-    The unittest framework is synchronous, so the test must be complete
-    by the time the test method returns.  This method provides the stop()
-    and wait() methods for this purpose.  The test method itself must call
-    self.wait(), and asynchronous callbacks should call self.stop() to signal
-    completion.
+    The unittest framework is synchronous, so the test must be
+    complete by the time the test method returns.  This class provides
+    the `stop()` and `wait()` methods for this purpose.  The test
+    method itself must call ``self.wait()``, and asynchronous
+    callbacks should call ``self.stop()`` to signal completion.
+    Alternately, the `gen_test` decorator can be used to use yield points
+    from the `tornado.gen` module.
 
-    By default, a new IOLoop is constructed for each test and is available
-    as self.io_loop.  This IOLoop should be used in the construction of
+    By default, a new `.IOLoop` is constructed for each test and is available
+    as ``self.io_loop``.  This `.IOLoop` should be used in the construction of
     HTTP clients/servers, etc.  If the code being tested requires a
-    global IOLoop, subclasses should override get_new_ioloop to return it.
+    global `.IOLoop`, subclasses should override `get_new_ioloop` to return it.
 
-    The IOLoop's start and stop methods should not be called directly.
-    Instead, use self.stop self.wait.  Arguments passed to self.stop are
-    returned from self.wait.  It is possible to have multiple
-    wait/stop cycles in the same test.
+    The `.IOLoop`'s ``start`` and ``stop`` methods should not be
+    called directly.  Instead, use `self.stop <stop>` and `self.wait
+    <wait>`.  Arguments passed to ``self.stop`` are returned from
+    ``self.wait``.  It is possible to have multiple ``wait``/``stop``
+    cycles in the same test.
 
     Example::
 
-        # This test uses an asynchronous style similar to most async
-        # application code.
+        # This test uses argument passing between self.stop and self.wait.
         class MyTestCase(AsyncTestCase):
+            def test_http_fetch(self):
+                client = AsyncHTTPClient(self.io_loop)
+                client.fetch("http://www.tornadoweb.org/", self.stop)
+                response = self.wait()
+                # Test contents of response
+                self.assertIn("FriendFeed", response.body)
+
+        # This test uses an explicit callback-based style.
+        class MyTestCase2(AsyncTestCase):
             def test_http_fetch(self):
                 client = AsyncHTTPClient(self.io_loop)
                 client.fetch("http://www.tornadoweb.org/", self.handle_fetch)
@@ -91,19 +131,6 @@ class AsyncTestCase(unittest.TestCase):
                 # self.wait() in test_http_fetch() via stack_context.
                 self.assertIn("FriendFeed", response.body)
                 self.stop()
-
-        # This test uses the argument passing between self.stop and self.wait
-        # for a simpler, more synchronous style.
-        # This style is recommended over the preceding example because it
-        # keeps the assertions in the test method itself, and is therefore
-        # less sensitive to the subtleties of stack_context.
-        class MyTestCase2(AsyncTestCase):
-            def test_http_fetch(self):
-                client = AsyncHTTPClient(self.io_loop)
-                client.fetch("http://www.tornadoweb.org/", self.stop)
-                response = self.wait()
-                # Test contents of response
-                self.assertIn("FriendFeed", response.body)
     """
     def __init__(self, *args, **kwargs):
         super(AsyncTestCase, self).__init__(*args, **kwargs)
@@ -116,31 +143,35 @@ class AsyncTestCase(unittest.TestCase):
     def setUp(self):
         super(AsyncTestCase, self).setUp()
         self.io_loop = self.get_new_ioloop()
+        self.io_loop.make_current()
 
     def tearDown(self):
+        self.io_loop.clear_current()
         if (not IOLoop.initialized() or
-            self.io_loop is not IOLoop.instance()):
+                self.io_loop is not IOLoop.instance()):
             # Try to clean up any file descriptors left open in the ioloop.
             # This avoids leaks, especially when tests are run repeatedly
             # in the same process with autoreload (because curl does not
             # set FD_CLOEXEC on its file descriptors)
             self.io_loop.close(all_fds=True)
         super(AsyncTestCase, self).tearDown()
+        # In case an exception escaped or the StackContext caught an exception
+        # when there wasn't a wait() to re-raise it, do so here.
+        # This is our last chance to raise an exception in a way that the
+        # unittest machinery understands.
+        self.__rethrow()
 
     def get_new_ioloop(self):
-        '''Creates a new IOLoop for this test.  May be overridden in
-        subclasses for tests that require a specific IOLoop (usually
-        the singleton).
-        '''
+        """Creates a new `.IOLoop` for this test.  May be overridden in
+        subclasses for tests that require a specific `.IOLoop` (usually
+        the singleton `.IOLoop.instance()`).
+        """
         return IOLoop()
 
-    @contextlib.contextmanager
-    def _stack_context(self):
-        try:
-            yield
-        except Exception:
-            self.__failure = sys.exc_info()
-            self.stop()
+    def _handle_exception(self, typ, value, tb):
+        self.__failure = sys.exc_info()
+        self.stop()
+        return True
 
     def __rethrow(self):
         if self.__failure is not None:
@@ -149,19 +180,21 @@ class AsyncTestCase(unittest.TestCase):
             raise_exc_info(failure)
 
     def run(self, result=None):
-        with StackContext(self._stack_context):
+        with ExceptionStackContext(self._handle_exception):
             super(AsyncTestCase, self).run(result)
-        # In case an exception escaped super.run or the StackContext caught
-        # an exception when there wasn't a wait() to re-raise it, do so here.
+        # As a last resort, if an exception escaped super.run() and wasn't
+        # re-raised in tearDown, raise it here.  This will cause the
+        # unittest run to fail messily, but that's better than silently
+        # ignoring an error.
         self.__rethrow()
 
     def stop(self, _arg=None, **kwargs):
-        '''Stops the ioloop, causing one pending (or future) call to wait()
+        """Stops the `.IOLoop`, causing one pending (or future) call to `wait()`
         to return.
 
-        Keyword arguments or a single positional argument passed to stop() are
-        saved and will be returned by wait().
-        '''
+        Keyword arguments or a single positional argument passed to `stop()` are
+        saved and will be returned by `wait()`.
+        """
         assert _arg is None or not kwargs
         self.__stop_args = kwargs or _arg
         if self.__running:
@@ -170,36 +203,33 @@ class AsyncTestCase(unittest.TestCase):
         self.__stopped = True
 
     def wait(self, condition=None, timeout=5):
-        """Runs the IOLoop until stop is called or timeout has passed.
+        """Runs the `.IOLoop` until stop is called or timeout has passed.
 
         In the event of a timeout, an exception will be thrown.
 
-        If condition is not None, the IOLoop will be restarted after stop()
-        until condition() returns true.
+        If ``condition`` is not None, the `.IOLoop` will be restarted
+        after `stop()` until ``condition()`` returns true.
         """
         if not self.__stopped:
             if timeout:
                 def timeout_func():
                     try:
                         raise self.failureException(
-                          'Async operation timed out after %s seconds' %
-                          timeout)
+                            'Async operation timed out after %s seconds' %
+                            timeout)
                     except Exception:
                         self.__failure = sys.exc_info()
                     self.stop()
-                if self.__timeout is not None:
-                    self.io_loop.remove_timeout(self.__timeout)
-                self.__timeout = self.io_loop.add_timeout(time.time() + timeout, timeout_func)
+                self.__timeout = self.io_loop.add_timeout(self.io_loop.time() + timeout, timeout_func)
             while True:
                 self.__running = True
-                with NullContext():
-                    # Wipe out the StackContext that was established in
-                    # self.run() so that all callbacks executed inside the
-                    # IOLoop will re-run it.
-                    self.io_loop.start()
+                self.io_loop.start()
                 if (self.__failure is not None or
-                    condition is None or condition()):
+                        condition is None or condition()):
                     break
+            if self.__timeout is not None:
+                self.io_loop.remove_timeout(self.__timeout)
+                self.__timeout = None
         assert self.__stopped
         self.__stopped = False
         self.__rethrow()
@@ -209,11 +239,11 @@ class AsyncTestCase(unittest.TestCase):
 
 
 class AsyncHTTPTestCase(AsyncTestCase):
-    '''A test case that starts up an HTTP server.
+    """A test case that starts up an HTTP server.
 
-    Subclasses must override get_app(), which returns the
-    tornado.web.Application (or other HTTPServer callback) to be tested.
-    Tests will typically use the provided self.http_client to fetch
+    Subclasses must override `get_app()`, which returns the
+    `tornado.web.Application` (or other `.HTTPServer` callback) to be tested.
+    Tests will typically use the provided ``self.http_client`` to fetch
     URLs from this server.
 
     Example::
@@ -230,15 +260,16 @@ class AsyncHTTPTestCase(AsyncTestCase):
                 self.http_client.fetch(self.get_url('/'), self.stop)
                 response = self.wait()
                 # test contents of response
-    '''
+    """
     def setUp(self):
         super(AsyncHTTPTestCase, self).setUp()
-        self.__port = None
+        sock, port = bind_unused_port()
+        self.__port = port
 
         self.http_client = self.get_http_client()
         self._app = self.get_app()
         self.http_server = self.get_http_server()
-        self.http_server.listen(self.get_http_port(), address="127.0.0.1")
+        self.http_server.add_sockets([sock])
 
     def get_http_client(self):
         return AsyncHTTPClient(io_loop=self.io_loop)
@@ -249,17 +280,17 @@ class AsyncHTTPTestCase(AsyncTestCase):
 
     def get_app(self):
         """Should be overridden by subclasses to return a
-        tornado.web.Application or other HTTPServer callback.
+        `tornado.web.Application` or other `.HTTPServer` callback.
         """
         raise NotImplementedError()
 
     def fetch(self, path, **kwargs):
         """Convenience method to synchronously fetch a url.
 
-        The given path will be appended to the local server's host and port.
-        Any additional kwargs will be passed directly to
-        AsyncHTTPClient.fetch (and so could be used to pass method="POST",
-        body="...", etc).
+        The given path will be appended to the local server's host and
+        port.  Any additional kwargs will be passed directly to
+        `.AsyncHTTPClient.fetch` (and so could be used to pass
+        ``method="POST"``, ``body="..."``, etc).
         """
         self.http_client.fetch(self.get_url(path), self.stop, **kwargs)
         return self.wait()
@@ -275,8 +306,6 @@ class AsyncHTTPTestCase(AsyncTestCase):
 
         A new port is chosen for each test.
         """
-        if self.__port is None:
-            self.__port = get_unused_port()
         return self.__port
 
     def get_protocol(self):
@@ -289,7 +318,9 @@ class AsyncHTTPTestCase(AsyncTestCase):
 
     def tearDown(self):
         self.http_server.stop()
-        self.http_client.close()
+        if (not IOLoop.initialized() or
+                self.http_client.io_loop is not IOLoop.instance()):
+            self.http_client.close()
         super(AsyncHTTPTestCase, self).tearDown()
 
 
@@ -301,7 +332,8 @@ class AsyncHTTPSTestCase(AsyncHTTPTestCase):
     def get_http_client(self):
         # Some versions of libcurl have deadlock bugs with ssl,
         # so always run these tests with SimpleAsyncHTTPClient.
-        return SimpleAsyncHTTPClient(io_loop=self.io_loop, force_instance=True)
+        return SimpleAsyncHTTPClient(io_loop=self.io_loop, force_instance=True,
+                                     defaults=dict(validate_cert=False))
 
     def get_httpserver_options(self):
         return dict(ssl_options=self.get_ssl_options())
@@ -315,15 +347,39 @@ class AsyncHTTPSTestCase(AsyncHTTPTestCase):
         # openssl req -new -keyout tornado/test/test.key -out tornado/test/test.crt -nodes -days 3650 -x509
         module_dir = os.path.dirname(__file__)
         return dict(
-                certfile=os.path.join(module_dir, 'test', 'test.crt'),
-                keyfile=os.path.join(module_dir, 'test', 'test.key'))
+            certfile=os.path.join(module_dir, 'test', 'test.crt'),
+            keyfile=os.path.join(module_dir, 'test', 'test.key'))
 
     def get_protocol(self):
         return 'https'
 
-    def fetch(self, path, **kwargs):
-        return AsyncHTTPTestCase.fetch(self, path, validate_cert=False,
-                   **kwargs)
+
+def gen_test(f):
+    """Testing equivalent of ``@gen.coroutine``, to be applied to test methods.
+
+    ``@gen.coroutine`` cannot be used on tests because the `.IOLoop` is not
+    already running.  ``@gen_test`` should be applied to test methods
+    on subclasses of `AsyncTestCase`.
+
+    Example::
+
+        class MyTest(AsyncHTTPTestCase):
+            @gen_test
+            def test_something(self):
+                response = yield gen.Task(self.fetch('/'))
+
+    """
+    f = gen.coroutine(f)
+
+    @functools.wraps(f)
+    def wrapper(self):
+        return self.io_loop.run_sync(functools.partial(f, self), timeout=5)
+    return wrapper
+
+
+# Without this attribute, nosetests will try to run gen_test as a test
+# anywhere it is imported.
+gen_test.__test__ = False
 
 
 class LogTrapTestCase(unittest.TestCase):
@@ -334,29 +390,29 @@ class LogTrapTestCase(unittest.TestCase):
     the test succeeds, so this class can be useful to minimize the noise.
     Simply use it as a base class for your test case.  It is safe to combine
     with AsyncTestCase via multiple inheritance
-    ("class MyTestCase(AsyncHTTPTestCase, LogTrapTestCase):")
+    (``class MyTestCase(AsyncHTTPTestCase, LogTrapTestCase):``)
 
-    This class assumes that only one log handler is configured and that
-    it is a StreamHandler.  This is true for both logging.basicConfig
-    and the "pretty logging" configured by tornado.options.
+    This class assumes that only one log handler is configured and
+    that it is a `~logging.StreamHandler`.  This is true for both
+    `logging.basicConfig` and the "pretty logging" configured by
+    `tornado.options`.  It is not compatible with other log buffering
+    mechanisms, such as those provided by some test runners.
     """
     def run(self, result=None):
         logger = logging.getLogger()
-        if len(logger.handlers) > 1:
-            # Multiple handlers have been defined.  It gets messy to handle
-            # this, especially since the handlers may have different
-            # formatters.  Just leave the logging alone in this case.
-            super(LogTrapTestCase, self).run(result)
-            return
         if not logger.handlers:
             logging.basicConfig()
-        self.assertEqual(len(logger.handlers), 1)
         handler = logger.handlers[0]
-        assert isinstance(handler, logging.StreamHandler)
+        if (len(logger.handlers) > 1 or
+                not isinstance(handler, logging.StreamHandler)):
+            # Logging has been configured in a way we don't recognize,
+            # so just leave it alone.
+            super(LogTrapTestCase, self).run(result)
+            return
         old_stream = handler.stream
         try:
             handler.stream = StringIO()
-            logging.info("RUNNING TEST: " + str(self))
+            gen_log.info("RUNNING TEST: " + str(self))
             old_error_count = len(result.failures) + len(result.errors)
             super(LogTrapTestCase, self).run(result)
             new_error_count = len(result.failures) + len(result.errors)
@@ -366,6 +422,50 @@ class LogTrapTestCase(unittest.TestCase):
             handler.stream = old_stream
 
 
+class ExpectLog(logging.Filter):
+    """Context manager to capture and suppress expected log output.
+
+    Useful to make tests of error conditions less noisy, while still
+    leaving unexpected log entries visible.  *Not thread safe.*
+
+    Usage::
+
+        with ExpectLog('tornado.application', "Uncaught exception"):
+            error_response = self.fetch("/some_page")
+    """
+    def __init__(self, logger, regex, required=True):
+        """Constructs an ExpectLog context manager.
+
+        :param logger: Logger object (or name of logger) to watch.  Pass
+            an empty string to watch the root logger.
+        :param regex: Regular expression to match.  Any log entries on
+            the specified logger that match this regex will be suppressed.
+        :param required: If true, an exeption will be raised if the end of
+            the ``with`` statement is reached without matching any log entries.
+        """
+        if isinstance(logger, basestring_type):
+            logger = logging.getLogger(logger)
+        self.logger = logger
+        self.regex = re.compile(regex)
+        self.required = required
+        self.matched = False
+
+    def filter(self, record):
+        message = record.getMessage()
+        if self.regex.match(message):
+            self.matched = True
+            return False
+        return True
+
+    def __enter__(self):
+        self.logger.addFilter(self)
+
+    def __exit__(self, typ, value, tb):
+        self.logger.removeFilter(self)
+        if not typ and self.required and not self.matched:
+            raise Exception("did not get expected log message")
+
+
 def main(**kwargs):
     """A simple test runner.
 
@@ -381,10 +481,11 @@ def main(**kwargs):
     be specified.
 
     Projects with many tests may wish to define a test script like
-    tornado/test/runtests.py.  This script should define a method all()
-    which returns a test suite and then call tornado.testing.main().
-    Note that even when a test script is used, the all() test suite may
-    be overridden by naming a single test on the command line::
+    ``tornado/test/runtests.py``.  This script should define a method
+    ``all()`` which returns a test suite and then call
+    `tornado.testing.main()`.  Note that even when a test script is
+    used, the ``all()`` test suite may be overridden by naming a
+    single test on the command line::
 
         # Runs all tests
         python -m tornado.test.runtests
@@ -399,25 +500,37 @@ def main(**kwargs):
     """
     from tornado.options import define, options, parse_command_line
 
-    define('autoreload', type=bool, default=False,
-           help="DEPRECATED: use tornado.autoreload.main instead")
-    define('httpclient', type=str, default=None)
     define('exception_on_interrupt', type=bool, default=True,
            help=("If true (default), ctrl-c raises a KeyboardInterrupt "
                  "exception.  This prints a stack trace but cannot interrupt "
                  "certain operations.  If false, the process is more reliably "
                  "killed, but does not print a stack trace."))
-    argv = [sys.argv[0]] + parse_command_line(sys.argv)
 
-    if options.httpclient:
-        from tornado.httpclient import AsyncHTTPClient
-        AsyncHTTPClient.configure(options.httpclient)
+    # support the same options as unittest's command-line interface
+    define('verbose', type=bool)
+    define('quiet', type=bool)
+    define('failfast', type=bool)
+    define('catch', type=bool)
+    define('buffer', type=bool)
+
+    argv = [sys.argv[0]] + parse_command_line(sys.argv)
 
     if not options.exception_on_interrupt:
         signal.signal(signal.SIGINT, signal.SIG_DFL)
 
+    if options.verbose is not None:
+        kwargs['verbosity'] = 2
+    if options.quiet is not None:
+        kwargs['verbosity'] = 0
+    if options.failfast is not None:
+        kwargs['failfast'] = True
+    if options.catch is not None:
+        kwargs['catchbreak'] = True
+    if options.buffer is not None:
+        kwargs['buffer'] = True
+
     if __name__ == '__main__' and len(argv) == 1:
-        print >> sys.stderr, "No tests specified"
+        print("No tests specified", file=sys.stderr)
         sys.exit(1)
     try:
         # In order to be able to run tests by their fully-qualified name
@@ -430,16 +543,12 @@ def main(**kwargs):
             unittest.main(module=None, argv=argv, **kwargs)
         else:
             unittest.main(defaultTest="all", argv=argv, **kwargs)
-    except SystemExit, e:
+    except SystemExit as e:
         if e.code == 0:
-            logging.info('PASS')
+            gen_log.info('PASS')
         else:
-            logging.error('FAIL')
-        if not options.autoreload:
-            raise
-    if options.autoreload:
-        import tornado.autoreload
-        tornado.autoreload.wait()
+            gen_log.error('FAIL')
+        raise
 
 if __name__ == '__main__':
     main()
